@@ -2,6 +2,9 @@
 
 require_once(NUE_CONSTANTS);
 require_once(NUE_LIB_ROOT . '/dao/CUser.php');
+require_once(NUE_LIB_ROOT . '/dao/CTopic.php');
+require_once(NUE_LIB_ROOT . '/dao/CTagTree.php');
+require_once(NUE_LIB_ROOT . '/util/CPager.php');
 
 // TODO : これそろそろ分割考えたほうがいいんじゃねえの？
 
@@ -11,9 +14,6 @@ require_once(NUE_LIB_ROOT . '/dao/CUser.php');
 class CDocumentBuilder
 {
 
-	/**	生のXMLを出力するかどうか。 */
-	const DEBUG_OUTPUT_RAW_XML = false;
-
 	/**	XML名前空間URL。 */
 	const URI_XMLNS = 'http://www.w3.org/2000/xmlns/';
 
@@ -44,19 +44,25 @@ class CDocumentBuilder
 	/**	XMLルートのタイトル属性。 */
 	private $title;
 
+	/**
+	 *	ルート要素を作成します。
+	 *
+	 *	@param DOMDocument $dom DOMオブジェクト。
+	 *	@return DOMElement ルート要素。
+	 */
 	public static function createBody(DOMDocument $dom)
 	{
-		$body = $dom->createElement('body');
-		$body->setAttributeNS(self::URI_XMLNS , 'xmlns:' . self::NS_XHTML, self::URI_XHTML);
-		$body->setAttributeNS(self::URI_XMLNS , 'xmlns:' . self::NS_XSI, self::URI_XSI);
-		$dom->appendChild($body);
-		return $body;
+		$result = $dom->createElement('body');
+		$result->setAttributeNS(self::URI_XMLNS , 'xmlns:' . self::NS_XHTML, self::URI_XHTML);
+		$result->setAttributeNS(self::URI_XMLNS , 'xmlns:' . self::NS_XSI, self::URI_XSI);
+		$dom->appendChild($result);
+		return $result;
 	}
 
 	/**
 	 *	コンストラクタ。
 	 *
-	 *	param string $title タイトル メッセージ。
+	 *	@param string $title タイトル メッセージ。
 	 */
 	public function __construct($title = '')
 	{
@@ -67,7 +73,7 @@ class CDocumentBuilder
 		$this->createAttribute($body, 'site', CConfigure::SITE_NAME);
 		$this->createAttribute($body, 'ver', CConstants::VERSION);
 		$this->createAttribute($body, 'ua', $_SERVER['HTTP_USER_AGENT']);
-		$body->setAttributeNS(self::URI_XSI, self::NS_XSI . ':noNamespaceSchemaLocation', './skin/nue.xsd');
+		$body->setAttributeNS(self::URI_XSI, self::NS_XSI . ':noNamespaceSchemaLocation', 'skin/nue.xsd');
 		$this->body = $body;
 		$this->title = $title;
 	}
@@ -126,11 +132,19 @@ class CDocumentBuilder
 			$p = $this->createParagraph($topic);
 			$this->addText($p, $_GET['err']);
 		}
+		if(strlen(self::$trace) > 0)
+		{
+			$this->createCodeParagraph($this->createTopic(_('デバッグ用メッセージ')), self::$trace);
+		}
 		ob_start("ob_gzhandler");
-		if(self::DEBUG_OUTPUT_RAW_XML)
+		if(CConfigure::USE_CLIENT_XSLT)
 		{
 			header('Content-Type: text/xml; charset=UTF-8');
-			echo $this->getDOM()->saveXML();
+			$dom = $this->getDOM();
+			$xsl = $dom->createProcessingInstruction('xml-stylesheet',
+				sprintf('type="text/xsl" href="skin/%s/%s"', CConfigure::SKINSET, $xslpath));
+			$dom->insertBefore($xsl, $dom->firstChild);
+			echo $dom->saveXML();
 		}
 		else
 		{
@@ -152,10 +166,6 @@ class CDocumentBuilder
 	 */
 	public function createHTML($xslpath)
 	{
-		if(strlen(self::$trace) > 0)
-		{
-			$this->createCodeParagraph($this->createTopic(_('デバッグ用メッセージ')), self::$trace);
-		}
 		$xslt = new XSLTProcessor();
 		$xsl = new DOMDocument();
 		$xsl->load(sprintf('%s/skin/%s/%s', NUE_ROOT, CConfigure::SKINSET, $xslpath));
@@ -163,6 +173,22 @@ class CDocumentBuilder
 		return $xslt->transformToXML($this->getDOM());
 	}
 
+	/**
+	 *	カテゴリリストを作成します。
+	 *
+	 *	@return DOMElement カテゴリ一覧情報 オブジェクト。
+	 */
+	public function createCategoryList()
+	{
+		$dom = $this->getDOM();
+		$result = $dom->createElement('category');
+		$tree = new CTagTree();
+		$this->getRootElement()->appendChild($result);
+		$this->createCategoryListChild($result, $tree->getTree());
+		$this->getRootElement()->appendChild($result);
+		return $result;
+	}
+
 	/**
 	 *	ユーザ情報を作成します。
 	 *
@@ -176,27 +202,104 @@ class CDocumentBuilder
 		if($user !== null)
 		{
 			$body =& $user->storage();
-			$result = $this->getDOM()->createElement('user');
 			if($enableLogoff)
 			{
 				$this->createAttribute($result, 'id', $user->getID());
 			}
 			$this->createAttribute($result, 'name', $body['name']);
+			if($body['root'])
+			{
+				$this->createAttribute($result, 'root', 'root');
+			}
 		}
 		$this->getRootElement()->appendChild($result);
 		return $result;
 	}
 
+	/**
+	 *	検索ワード情報を作成します。
+	 *
+	 *	@param string $expr 検索ワード。
+	 *	@return DOMElement ユーザ情報 オブジェクト。
+	 */
+	public function createSearchInfo($expr = null)
+	{
+		$result = $this->getDOM()->createElement('search');
+		if($expr !== null && strlen($expr) > 0)
+		{
+			$this->createAttribute($result, 'tag', $expr);
+		}
+		$this->getRootElement()->appendChild($result);
+		return $result;
+	}
+
+	/**
+	 *	ページャ情報を作成します。
+	 *
+	 *	@param CPager $pager ページャ オブジェクト。
+	 *	@return DOMElement ユーザ情報 オブジェクト。
+	 */
+	public function createPagerInfo(CPager $pager = null)
+	{
+		if($pager === null)
+		{
+			$pager = new CPager();
+		}
+		$result = $this->getDOM()->createElement('pager');
+		$this->createAttribute($result, 'page', $pager->target);
+		$this->createAttribute($result, 'tpp', $pager->TopicsPerPage);
+		$this->createAttribute($result, 'max', $pager->maxPage);
+		$this->getRootElement()->appendChild($result);
+		return $result;
+	}
+
 	/**
 	 *	空のトピックを作成します。
 	 *
-	 *	@param string $caption 見出し。
+	 *	@param mixed $caption 見出し、または記事DAOオブジェクト。
 	 *	@return DOMElement 空のトピック オブジェクト。
 	 */
 	public function createTopic($caption)
 	{
-		$topic = $this->getDOM()->createElement('topic');
-		$this->createAttribute($topic, 'title', $caption);
+		$dom = $this->getDOM();
+		$topic = $dom->createElement('topic');
+		$title = $caption;
+		if($caption instanceof CTopic)
+		{
+			$entity = $caption->getEntity();
+			$body =& $entity->storage();
+			$title = $body['caption'];
+			$this->createAttribute($topic, 'id', $caption->getID());
+			$this->createAttribute($topic, 'updated', date('Y/n/j', $entity->getUpdated()));
+			$this->createAttribute($topic, 'created', date('Y/n/j', $caption->userTimeStamp));
+			$ul = null;
+			foreach($caption->getDescription() as $desc)
+			{
+				if(preg_match('/^@/', $desc))
+				{
+					$separate = preg_match('/^@@/', $desc);
+					if($separate)
+					{
+						$ul = null;
+					}
+					if($ul === null)
+					{
+						$ul = $dom->createElement('ul');
+						$topic->appendChild($ul);
+					}
+					$li = $dom->createElement('li');
+					$ul->appendChild($li);
+					$this->addHLML($li, substr($desc, $separate ? 2 : 1), $ul);
+				}
+				else
+				{
+					$ul = null;
+					$p = $this->createParagraph($topic);
+					$this->addHLML($p, $desc);
+				}
+			}
+		}
+		$this->createAttribute($topic, 'title', $title);
 		$this->getRootElement()->appendChild($topic);
 		return $topic;
 	}
@@ -338,38 +441,44 @@ class CDocumentBuilder
 	 *
 	 *	@param DOMNode $element 所属させる要素。
 	 *	@param string $expr 文字列。
+	 *	@param string $paragraph 一番親となる要素。
 	 */
-	public function addHLML(DOMNode $element, $expr)
+	public function addHLML(DOMNode $element, $expr, DOMNode $paragraph = null)
 	{
-		$dom = $this->getDOM();
 		$result = array();
-		if(preg_match('/^(.*?)\[\[\[([a-zA-Z_]+?):(.*?)\](.*?)\]\](.*)$/',
-			$expr, $elm, PREG_OFFSET_CAPTURE))
+		if(preg_match('/\[((?>[^[\]]+)|(?R))*\]/', $expr, $match, PREG_OFFSET_CAPTURE))
 		{
-			$this->addText($element, $elm[1][0]);
-			$tag = $elm[2][0];
-			$elm[3][0] = preg_replace('/\\\,/', "\1\x00", $elm[3][0]);
-			$attrs = preg_split('/\,/', $elm[3][0], -1, PREG_SPLIT_NO_EMPTY);
-			for($i = count($attrs); --$i >= 0; )
+			$this->addText($element, substr($expr, 0, $match[0][1]));
+			if(preg_match('/\[\[\[(.*?):(.*?)\](.*)\]\]/', $match[0][0], $elm))
 			{
-				$attrs[$i] = preg_replace('/\x00/', ',', $attrs[$i]);
-				$attrs[$i] = preg_split('/=/', $attrs[$i], 2);
+				$tag = $elm[1];
+				$inner = $elm[3];
+				$result = $element;
+				$target = $this->getHLMLPath($tag);
+				if(file_exists($target))
+				{
+					$elm[2] = preg_replace('/\\\,/', "\x00", $elm[2]);
+					$attrs_base = preg_split('/\,/', $elm[2], -1, PREG_SPLIT_NO_EMPTY);
+					$attrs = array();
+					for($i = count($attrs_base); --$i >= 0; )
+					{
+						$kv = preg_split('/=/', preg_replace('/\x00/', ',', $attrs_base[$i]), 2);
+						$attrs[$kv[0]] = count($kv) == 2 ? $kv[1] : null;
+					}
+					if($paragraph === null)
+					{
+						$paragraph = $element;
+					}
+					$dom = $this->getDOM();
+					require($target);
+					if($result !== $element)
+					{
+						$element->appendChild($result);
+					}
+				}
+				$this->addHLML($result, $inner, $paragraph);
 			}
-			$inner = $elm[4][0];
-			$expr = substr($expr, $elm[5][1]);
-			$result = $element;
-			$target = $this->getHLMLPath($tag);
-			$exists = file_exists($target);
-			if($exists)
-			{
-				require_once($target);
-			}
-			if($exists && $result !== $element)
-			{
-				$element->appendChild($result);
-			}
-			$this->addHLML($result, $inner);
-			$this->addHLML($element, $expr);
+			$this->addHLML($element, substr($expr, $match[0][1] + strlen($match[0][0])), $paragraph);
 		}
 		else
 		{
@@ -393,6 +502,63 @@ class CDocumentBuilder
 		return $attr;
 	}
 
+	/**
+	 *	隠し入力を作成します。
+	 *
+	 *	@param DOMNode $parent 所属させるノード。
+	 *	@param string $id キーとなる文字列。
+	 *	@param string $value 既定値となる文字列。
+	 *	@return DOMElement input要素オブジェクト。
+	 */
+	public function createHiddenInput(
+		DOMNode $parent, $id, $value)
+	{
+		return $this->createHTMLElement($parent, 'input', array(
+			'type' => 'hidden',
+			'id' => $id, 'name' => $id,
+			'value' => $value));
+	}
+
+	/**
+	 *	送信ボタンを作成します。
+	 *
+	 *	@param DOMNode $parent 所属させるノード。
+	 *	@param string $caption ボタン名文字列。
+	 *	@return DOMElement input要素オブジェクト。
+	 */
+	public function createSubmitInput(
+		DOMNode $parent, $caption)
+	{
+		return $this->createHTMLElement($parent, 'input', array(
+			'type' => 'submit', 'value' => $caption));
+	}
+
+	/**
+	 *	チェック ボックスを作成します。
+	 *
+	 *	@param DOMNode $parent 所属させるノード。
+	 *	@param string $id キーとなる文字列。
+	 *	@param boolean $value 有効かどうか。
+	 *	@param string $label ラベル。
+	 *	@return DOMElement input要素オブジェクト。
+	 */
+	public function createCheckInput(
+		DOMNode $parent, $id, $value, $label)
+	{
+		$this->createHTMLElement($parent, 'label', array('for' => $id),
+			$label);
+		$result = $this->createHTMLElement($parent, 'input', array(
+			'type' => 'checkbox',
+			'id' => $id, 'name' => $id,
+			'value' => 1));
+		if($value)
+		{
+			$this->createAttribute($result, 'checked', 'checked');
+		}
+		$this->createHTMLElement($parent, 'br');
+		return $result;
+	}
+
 	/**
 	 *	1行入力ボックスを作成します。
 	 *
@@ -403,16 +569,25 @@ class CDocumentBuilder
 	 *	@param string $label ラベル。
 	 *	@param integer $min 最小文字数。
 	 *	@param integer $max 最大文字数。
-	 *	@param boolean $ascii ASCII入力のみ受け付けるかどうか。
+	 *	@param mixed $ascii ASCII入力のみ受け付けるかどうか。
 	 *	@param boolean $enabled 有効なフィールドかどうか。
 	 *	@return DOMElement input要素オブジェクト。
 	 */
 	public function createTextInput(
 		DOMNode $parent, $type, $id, $value, $label, $min = 0, $max = 255,
-		$ascii = true, $enabled = true)
+		$ascii = false, $enabled = true)
 	{
 		$this->createHTMLElement($parent, 'label', array('for' => $id),
 			$label);
+		$regex = '.';
+		if($ascii === true)
+		{
+			$regex = '[0-9A-Za-z]';
+		}
+		elseif($ascii === 0)
+		{
+			$regex = '[0-9]';
+		}
 		$result = $this->createHTMLElement($parent, 'input', array(
 			'type' => $type,
 			'id' => $id, 'name' => $id,
@@ -470,7 +645,7 @@ class CDocumentBuilder
 	 *	既定のHLML変換をします。
 	 *
 	 *	@param string $tag 要素名。
-	 *	@param array $attrs 要素一覧。[[key,value][key,value]...]
+	 *	@param array $attrs 属性一覧が格納された連想配列。
 	 *	@param array $allow 許容する要素一覧。(全許容する場合null、全禁止する場合空の配列)
 	 *	@return DOMElement 要素オブジェクト。
 	 */
@@ -483,15 +658,52 @@ class CDocumentBuilder
 		{
 			$allow = array_merge($allow, self::$hlmlAutoAllow);
 		}
-		foreach($attrs as $item)
+		foreach(array_keys($attrs) as $item)
 		{
-			if($allAllow || in_array($item[0], $allow))
+			if($allAllow || in_array($item, $allow))
 			{
-				$this->createAttribute($result, $item[0], $item[1]);
+				$this->createAttribute($result, $item, $attrs[$item]);
 			}
 		}
 		return $result;
 	}
+
+	/**
+	 *	カテゴリリストを作成します。
+	 *
+	 *	@param DOMNode $topic 所属させるノード。
+	 *	@param array $tree カテゴリ ツリー情報。
+	 */
+	private function createCategoryListChild(DOMNode $parent, array $tree)
+	{
+		$dom = $this->getDOM();
+		$keys = array_keys($tree);
+		if(count($keys) > 0)
+		{
+			$len = max($keys);
+			for($i = 0; $i <= $len; $i++)
+			{
+				$li = $dom->createElement('li');
+				$parent->appendChild($li);
+				$item = $tree[$i];
+				if(isset($tree[$item]))
+				{
+					$ul = $dom->createElement('ul');
+					$lh = $dom->createElement('lh');
+					$this->createAttribute($lh, 'href', urlencode($item));
+					$this->addText($lh, $item);
+					$li->appendChild($ul);
+					$ul->appendChild($lh);
+					$this->createCategoryListChild($ul, $tree[$item]);
+				}
+				else
+				{
+					$this->createAttribute($li, 'href', urlencode($item));
+					$this->addText($li, $item);
+				}
+			}
+		}
+	}
 }
 
 /**
@@ -501,7 +713,7 @@ class CDocumentBuilder
  */
 function trace($body)
 {
-	CDocumentBuilder::$trace .= "\n" . $body;
+	CDocumentBuilder::$trace .= "\n\n" . $body;
 }
 
 ?>
